/**************************************************************
 ***** Creates APPX Connector service process for Windows, *****
 ************** Linux, and Mac operating systems ***************
 **************************************************************/

/*
###############################################################################
###############################################################################
@usage: node appxConnector-install [appx | mongo] [appxport] [mongoport]
@example 1: node appxConnector-install appx 3014
@example 2: node appxConnector-install mongo 3015
@example 4: node appxConnector-install 3016 3017
@example 4: node appxConnector-install
###############################################################################
@Code defaults: if only 1 port is provided we will default port for 
appxConnector. If no first argument is provided we will default to install both
connectors.
###############################################################################
###############################################################################
*/

var serverConnectorInstallVersionStr = "5.4.5.16110211";
var serverConnectorInstallVersionNum = 50405.16110211;

var args = process.argv.slice(2);
var exec = require('child_process').execSync, child;
var services = [
    {
	name: 'appxConnector',
	description: 'Connector that allows appx client to talk to server.',
	script: 'appxConnector.js',
	
	//Set variable so appxConnector.js knows where to push files to.
	env:[{
	    name: "APPX_CONNECTOR_DIR",
	    value: require('path').resolve(__dirname)
	},{
	    name: "NODE_ENV",
	    value: "production"
	},{
	    name: "APPX_CONNECTOR_PORT",
	    value: 3014
	}
	   ]
    },
    {
	name: 'appxMongoConnector',
	description: 'Connector that allows browser to make requests to mongo database.',
	script: 'appxMongoConnector.js',
	//Set variable so appxConnector.js knows where to push files to.
	env:[{
	    name: "APPX_MONGO_CONNECTOR_DIR",
	    value: require('path').resolve(__dirname)
	},{
	    name: "NODE_ENV",
	    value: "production"
	},{
	    name: "APPX_MONGO_CONNECTOR_PORT",
	    value: 3015
	}
	    ]
    }
    
];

/*
**Function to process arguments provided by the user. If no arguments or
**incorrect arguments are provided then we default install.
*/
function processArgs(){
    if (args.length > 0){
	if ( !isNaN(args[0]) ){
	    if (args.length > 2){
		argError();
	    }
	    for (var i = 0; i < args.length; i++){
		services[i].env[2].value = args[i];
		services[i].name = services[i].name.concat(args[i]);
	    }
	}else{
	    processServiceArg();
	}
    }else{
	console.log("No arguments recieved, using default settings for install")
	for (var i = 0; i < services.length; i++){
	    services[i].name = services[i].name.concat(services[i].env[2].value);
	}
    }
    
}

/*
**Function to process service argument provided by the user. If incorrect 
**argument is provided then we default install.
*/
function processServiceArg(){
    if (args.length > 3){
	argError();
    }else{
	switch (args[0]){
	     case "appx":
		 if (args.length === 2){
		     services[0].env[2].value = args[1];
		     services[0].name = services[0].name.concat(args[1]);
		 }
		 services.splice(1,1);
		 break;
	     case "mongo":
		 if (args.length === 2){
		     services[1].env[2].value = args[1];
		     services[1].name = services[1].name.concat(args[1]);
		 }
		 services.splice(0,1);
		 break;
	     default:
		 for (var i = 1; i < args.length; i++){
		     services[i-1].env[2].value = args[i];
		     services[i-1].name = services[i-1].name.concat(args[i]);
		 }
		 console.log("Both services are being installed");
	}
    }
}

/*
**Function to throw error and abort program if to many arguments are provided with
**the command.  
*/
function argError(){
    try{
	throw "Invalid Argument";
    }catch (ex){
	console.log("Invalid function call. Check the examples and try again.");
    }
}

// Make sure the required modules are installed.
var modules = [
    'crypto',
    'fs',
    'hexy',
    'http',
    'https',
    'mime',
    'mongodb',
    'net',
    'node-cryptojs-aes',
    'string',
    'string_decoder',
    'url',
    'ws'
];

console.log("");
console.log("Checking for required modules...");

var allReqInstalled = true;
function moduleTest(){
    modules.forEach(function(name){
	if( ! testForModule(name) )
	    allReqInstalled = false;
    });

    console.log("");

    if( ! allReqInstalled ) {
	console.log("Missing modules.  Please install required modules and run again.");
	process.exit();
    }
}

//Detects OS and sets the Service variable accordingly.
switch(process.platform){
     case 'win32':
	 var Service = require('node-windows').Service; break;
     case 'linux':
	 var Service = require('node-linux').Service; break;
     case 'darwin':
	 var Service = require('node-mac').Service; break;
}

function installService(mService){
    //Create new service object
    var svc = new Service(services[mService]);

    //Installs service
    svc.on('alreadyinstalled', function(){
	console.log("Already installed, restarting "+services[mService].name+" service...");
	svc.stop();
	svc.start();
    });

    svc.on('install', function(){
	console.log("Installing "+services[mService].name+" service...");
	setTimeout(function(){
	    svc.start();
	},5000);
	
	console.log("Done... Waiting on service to start...");
    });

    svc.on('start', function(){
	console.log("Service "+services[mService].name+" started");
	if (++mService < services.length){
	    installService(mService);
	}else{
	    process.exit()
	}
	
    });

    svc.install();
}


function testForModule(name) {
    var nameStr = name + "                              ";
    process.stdout.write("Module: " + nameStr.substring(0,25) + " - ");
    if( isModuleInstalled(name) ) {
	console.log("Installed.");
	return true;
    }
    else {
	console.log("Missing, please run 'npm install " + name + "' to install.");
	return false;
    }
}

function isModuleInstalled(name) {
    try {
	require.resolve(name);
	return true;
    }
    catch(ex) {
	return false;
    }
}

function checkMongo(){
    var mongoDatabase    = "AppxDatabaseCache";
    var mongoHost        = "localhost";
    var mongoPort        = 27017;
    var MongoClient = require('mongodb').MongoClient;
    var MongoServer = require('mongodb').Server;
    try{
	var mongoClient = new MongoClient(new MongoServer(mongoHost, mongoPort));
	var mongoUrl    = 'mongodb://'+mongoHost+':'+mongoPort+'/'+mongoDatabase;
	var mongoCacheDb = null;

	mongoClient.connect(mongoUrl,function (err, db) {

	    mongoCacheDb = db;
	    if(err){
		console.log("Unable to connect to MongoDatabase Server. Make sure MongoDB server has been installed, configured, and started on your server before continuing. This is a separate setup from installing the driver using 'npm install mongodb'. You need to visit the website and follow the instructions for installing the server. (https://www.mongodb.org/downloads)");
		process.exit();
	    }
	    processArgs();
	    moduleTest();
	    installService(0);
	});

    }catch(ex){
	console.log(ex);
    }
}
checkMongo();
